package gr.ilsp.fmc.aligner.factory;

import static net.sourceforge.align.util.Util.merge;

import java.io.File;
import java.io.Writer;
import java.util.Date;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import net.sourceforge.align.coretypes.Alignment;
import net.sourceforge.align.formatter.Formatter;
import net.sourceforge.align.util.bind.TmxMarshallerUnmarshaller;
import net.sourceforge.align.util.bind.tmx.Body;
import net.sourceforge.align.util.bind.tmx.Header;
import net.sourceforge.align.util.bind.tmx.Prop;
import net.sourceforge.align.util.bind.tmx.Seg;
import net.sourceforge.align.util.bind.tmx.Tmx;
import net.sourceforge.align.util.bind.tmx.Tu;
import net.sourceforge.align.util.bind.tmx.Tuv;
import net.sourceforge.align.util.date.DateParser;

public class BilingualScoredTmxFormatter implements Formatter  {
	public static final String TMX_VERSION = "1.4b";

	public static final String TMX_ADMINLANG = "en";
	private static final Logger logger = LoggerFactory.getLogger(BilingualScoredTmxFormatter.class);

	public static final String TMX_CREATIONTOOL = "mALIGNa";

	public static final String TMX_CREATIONTOOLVERSION = "2";

	public static final String TMX_SEGTYPE = "block";

	public static final String TMX_DATATYPE = "plaintext";
	
	public static final String TMX_OTMF = "al";

	private String sourceLanguageCode;

	private String targetLanguageCode;

	private File sourceFile;

	private File targetFile;

	private Writer writer;

	public BilingualScoredTmxFormatter(Writer writer, String sourceLanguageCode, 
			String targetLanguageCode, File sourceFile, File targetFile) {
		this.writer = writer;
		this.sourceLanguageCode = sourceLanguageCode;
		this.targetLanguageCode = targetLanguageCode;
		this.sourceFile = sourceFile;
		this.targetFile = targetFile;
	}
	
	public void format(List<Alignment> alignmentList) {
		logger.debug("Formatting alignment list...");
		
		Tmx tmx = new Tmx();
		tmx.setVersion(TMX_VERSION);
		Header header = new Header();
		header.setAdminlang(TMX_ADMINLANG);
		header.setSrclang(sourceLanguageCode);
		header.setCreationtool(TMX_CREATIONTOOL);
		header.setCreationtoolversion(TMX_CREATIONTOOLVERSION);
		header.setSegtype(TMX_SEGTYPE);
		header.setDatatype(TMX_DATATYPE);
		header.setOTmf(TMX_OTMF);
		header.setCreationdate(DateParser.getIsoDateNoMillis(new Date()));

		Prop sourceFileProperty = new Prop();
		sourceFileProperty.setType("sourceFile");
		sourceFileProperty.getContent().add(sourceFile.getName());
		header.getNoteOrPropOrUde().add(sourceFileProperty);

		Prop targetFileProperty = new Prop();
		targetFileProperty.setType("targetFile");
		targetFileProperty.getContent().add(targetFile.getName());
		header.getNoteOrPropOrUde().add(targetFileProperty);

		Prop sourceLangProperty = new Prop();
		sourceLangProperty.setType("sourceLang");
		sourceLangProperty.getContent().add(sourceLanguageCode);
		header.getNoteOrPropOrUde().add(sourceLangProperty);

		Prop targetLangProperty = new Prop();
		targetLangProperty.setType("targetLang");
		targetLangProperty.getContent().add(targetLanguageCode);
		header.getNoteOrPropOrUde().add(targetLangProperty);

		tmx.setHeader(header);
		Body body = new Body();
		int tuid = 1;
		for (Alignment alignment : alignmentList) {
			Tu tu = new Tu();
			tu.setTuid(""+tuid++);
			Prop scoreProperty = new Prop();
			scoreProperty.setType("score");
			scoreProperty.getContent().add(""+alignment.getScore());
			tu.getNoteOrProp().add(scoreProperty);
			createTuv(tu, sourceLanguageCode, alignment.getSourceSegmentList());
			createTuv(tu, targetLanguageCode, alignment.getTargetSegmentList());
			if (tu.getTuv().size() > 0) {
				body.getTu().add(tu);
			}
		}
		tmx.setBody(body);
		TmxMarshallerUnmarshaller.getInstance().marshal(tmx, writer);
	}

	private void createTuv(Tu tu, String languageCode, 
			List<String> segmentList) {
		if (segmentList.size() > 0) {
			Tuv tuv = new Tuv();
			tuv.setLang(languageCode);
			Seg seg = new Seg();
			String segment = merge(segmentList);
			seg.getContent().add(segment);
			tuv.setSeg(seg);
			tu.getTuv().add(tuv);
		}
	}

	
}
