package gr.ilsp.fmc.classifier;

import gr.ilsp.fmc.datums.ClassifierDatum;
import gr.ilsp.fmc.datums.ExtendedParsedDatum;
//import gr.ilsp.fmc.main.Bitexts;
import gr.ilsp.fmc.utils.TopicTools;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Map;
import java.util.Map.Entry;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;
//import org.apache.tika.language.LanguageIdentifier;

import com.cybozu.labs.langdetect.Detector;
import com.cybozu.labs.langdetect.DetectorFactory;
import com.cybozu.labs.langdetect.LangDetectException;







@SuppressWarnings("serial")
public class Classifier implements Serializable{
	private static final Logger LOGGER = Logger.getLogger(Classifier.class);

	private double TITLE_WEIGHT = 10;
	private double KEYWORDS_WEIGHT = 4;
	private double META_WEIGHT = 2;
	private double CONTENT_WEIGHT = 1;
	//private double TOTABSCORE_TH = 0.0;
	//private double TOTRELSCORE_TH = 0.2;
	//private double SUBCLASSSCORE_TH = 0.2;
	//private double relcontentthr = 0.1;
	private String _targetLanguage;
	//private int _minTokensNumber = 200;		
	private int _minTokensNumber;
	private String[] _classes;
	private String[] _targetlangKeys;

	private ArrayList<String[]> _topic;

	private double _absthres;
	private double _relthres;
	private static int _min_uniq_terms;
	private boolean _keepBoiler = false;

	private int _max_depth;
	
	private static String langIdentified;

	public Classifier(String[] langKeys, String language, String[] classes, 
			ArrayList<String[]> topic, double abs_thres, double rel_thres, boolean keepBoiler, 
			int min_uniq_terms, int max_depth, int minTokensNumber){
		_targetLanguage = language;
		_classes = classes;
		_topic = topic;
		_absthres = abs_thres;
		_relthres = rel_thres;
		_keepBoiler  = keepBoiler;
		_min_uniq_terms = min_uniq_terms;
		_max_depth = max_depth;
		_targetlangKeys=langKeys;
		_minTokensNumber=minTokensNumber;
	}
	public ClassifierDatum classify(ExtendedParsedDatum parsedDatum) {
		//String lang = parsedDatum.getLanguage();
		//System.out.println(lang);
		//System.out.println("Classifying " + parsedDatum.getUrl());
		String title = parsedDatum.getTitle();
		Map<String,String> metaMap = parsedDatum.getParsedMeta();
		String keywords = "";
		String meta = "";
		//System.out.println(parsedDatum.getUrl());
		String content = parsedDatum.getParsedText().toLowerCase();
		//if (_keepBoiler & parsedDatum.getParsedMeta().get("Content-Type").contains("text/html"))  
		//	content = cleanContent(content);
		//System.out.println(content);
		if (_keepBoiler) content = cleanContent(content);

		StringTokenizer tkzr = new StringTokenizer(content);
		int length_in_tok=tkzr.countTokens();
		
		//System.out.println(length_in_tok);
		//LOGGER.error(_minTokensNumber);
		if (length_in_tok<_minTokensNumber){
			//LOGGER.info("CUT due to its small clean content length:"+parsedDatum.getUrl());
			return null;
		}
		String url = parsedDatum.getUrl();
		//if (url.contains("wikipedia"))
		//	return null;
		//		for (String s:metaMap.keySet()){
		//			if (s.equals("keywords"))
		//				keywords = metaMap.get(s);
		//			if (s.equals("description"))
		//				meta = metaMap.get(s);			
		//		}
		for (Entry<String, String> s:metaMap.entrySet()){
			if (s.getKey().equals("keywords"))
				keywords = s.getValue();
			if (s.getKey().equals("description"))
				meta = s.getValue();			
		}
		//guess language of the content
		if (_targetLanguage!=null){
			//LanguageIdentifier LangI=new LanguageIdentifier(content); 
			//langIdentified = LangI.getLanguage();

			Detector detector = null;			
			try {
				detector = DetectorFactory.create();
				detector.append(content);
				langIdentified = detector.detect();										
			} catch (LangDetectException e) {
				//LOGGER.error(e.getMessage());
			}


			//System.out.println(langIdentified);
			String[] langs = _targetLanguage.split(";");
			boolean match = false;
			//vpapa
			for (String lang:langs){
				if (langIdentified.equals(lang)){
					match = true;
					break;
				}
			}
			
			if (!match)
				return null;
		}
		if (_topic==null){
			return new ClassifierDatum(url, new String[0],new Double[0][0], 0.0, 0.0,length_in_tok);
			//return new ClassifierDatum(url, new String[0],new Double[0][0], 0.0, 0.0);
		}		
		if (title==null) title = "";

		Double[][] titleScores = rankText(title,TITLE_WEIGHT ,_topic,_classes,false);
		Double[][] keywordsScores=rankText(keywords,KEYWORDS_WEIGHT ,_topic,_classes,false);
		Double[][] metaScores=rankText(meta,META_WEIGHT ,_topic,_classes,false);		
		Double[][] contentScores=rankText(content,CONTENT_WEIGHT ,_topic,_classes, true);
		//System.out.println("Score:" + titleScores[titleScores.length-1][0] + " " +contentScores[contentScores.length-1][0]);
		if (contentScores==null)
			return null;
		//ClassifierDatum result=classifyText(titleScores,keywordsScores,metaScores,contentScores,
		//		TOTABSCORE_TH,TOTRELSCORE_TH,SUBCLASSSCORE_TH,_classes, url, length_in_tok);
		ClassifierDatum result=classifyText(titleScores,keywordsScores,metaScores,contentScores,
				_absthres,_relthres,_relthres,_classes, url, length_in_tok);
		//System.out.println(parsedDatum.getUrl() + " " + TOTABSCORE_TH);
		if (result==null)
			return null;
		double contentscore = contentScores[contentScores.length-1][0];
		double relcontentscore = contentScores[contentScores.length-1][1];
		//added for running without topic
		if (_absthres==0.0){
			_relthres=-0.1;
		}
		if (contentscore>=_absthres && relcontentscore>_relthres){//TODO relthres value????
			//ClassifierDatum result = new ClassifierDatum(url, new String[1],new String[1][2], 0.0, 0.0);
			//System.out.println(url +" STORED "+ contentscore +"\t"+ relcontentscore);
			return result;
		}
		else
			return null;
	}


	public static ClassifierDatum classifyText(Double[][] scores1, Double[][] scores2,
			Double[][] scores3, Double[][] scores4, double thr1, double thr2, 
			double thr3, String[] classes, String url, int length_in_tok) {
		//get the arrays with scores for each location, the thresholds and the sub-classes
		//Boolean pass=false;
		//System.out.println("scored: " + url);
		double norm_score=0; 
		Double total_score=0.0;
		Double total_relscore = 0.0;
		//add the scores
		Double[][] sums= scores1;
		for (int ii=0;ii<scores1.length;ii++){
			sums[ii][0] += scores2[ii][0]+ scores3[ii][0]+scores4[ii][0];
			sums[ii][1] += scores2[ii][1]+ scores3[ii][1]+scores4[ii][1];
		}
		//print the sums of the scores for checking 
		//System.out.println("SUMS");
		//for (int ii=0;ii<scores1.length;ii++){
		//	System.out.println(sums[ii][0]+"        "+ sums[ii][1]);
		//}
		//if the total absolute and relative scores are over the corresponding thresholds
		//calculate a normalized score per subclass and compare it with another threshold
		ArrayList<String> subclasses = new ArrayList<String>();
		ArrayList<Double[][]> subscores = new ArrayList<Double[][]>(); 
		//if (sums[sums.length-1][0]>thr1 && sums[sums.length-1][1]>thr2){
		//vpapa
		if (sums[sums.length-1][0]>=thr1 && sums[sums.length-1][1]>=thr2){
			//pass=true;
			for (int ii=0;ii<scores1.length-1;ii++){
				//norm_score =100 *sums[ii][1]/sums[scores1.length-1][1];
				norm_score = sums[ii][1]/sums[scores1.length-1][1];
				//if (norm_score>thr3){
				//vpapa
				if (norm_score>=thr3){	
					subclasses.add(classes[ii]);
					//System.out.println("Text is in the subclass " + classes[ii]+".");
					total_score += sums[ii][0];
					total_relscore += sums[ii][1];
					subscores.add(new Double[][]{new Double[] {sums[ii][0],sums[ii][1]}});
				}
			}
		}else{
			return null;
		}
		String[] subclasses1=new String[subclasses.size()];
		for (int i=0;i<subclasses.size();i++){
			subclasses1[i]= subclasses.get(i);
		}
		Double[][] subscores1 = new Double[subscores.size()][2];
		for (int i=0;i<subscores.size();i++){
			Double[][] temp=subscores.get(i);
			subscores1[i][0] = temp[0][0];
			subscores1[i][1] = temp[0][1];
		}
		//System.out.println("The total score is "+total_score);
		total_relscore=scores4[scores4.length-1][1]/2; //rel score is based on content only.
		total_relscore=(double) (Math.round(total_relscore*1000))/1000;
		ClassifierDatum result = new ClassifierDatum(url, subclasses1,subscores1, total_score, total_relscore,length_in_tok);
		//ClassifierDatum result = new ClassifierDatum(url, subclasses1,subscores1, total_score, total_relscore);
		return result;
	}


	/*public static Double[][] rankText(String str, double w,ArrayList<String[]> topic, String[] classes){
		//str: String to classify,  topic: list of triplets,  classes:list of classes
		//Returns an array of doubles with 2 columns. 
		//The number of rows are as many as the subclasses plus one (the "total"). 
		//The first contains the absolute score per subclass, while the second relative scores, 
		//i.e. the absolute scores divided by the number of words. 



		//stem the input str
		ArrayList<String> stems = null;
		try {
			stems = TopicTools.analyze(str, langIdentified);
		} catch (IOException e) {
			LOGGER.warn(e.getMessage());
			return null;
		}

		//concatenate stems 
		str="";
    	for (String s:stems){ str=str.concat(" "+s);}
    	str = str.trim();
    	//System.out.println(str);
		//word counting
		StringTokenizer st = new StringTokenizer(str);	
		double words_num = 0;// String ss="";
		words_num = st.countTokens();
		//while (st.hasMoreTokens()) { ss= st.nextToken();words_num++;}
		//initialization of scores array
		Double[][] scores = new Double[classes.length+1][2];
		for (int ii=0;ii<classes.length;ii++){
			scores[ii][0] = 0.0;	scores[ii][1] = 0.0;
		}		
		scores[classes.length][0]=0.0;  scores[classes.length][1]=0.0;
		if (words_num==0) return scores;
		//lists with the found terms
		ArrayList<String> termfound=new ArrayList<String>(); 
		ArrayList<String> termfoundid=new ArrayList<String>();
		//calculate the initial scores based on the occurrences of each term of the topic 
		//in the str and the weight of the term
		String[] tempstr;		String term, term_class;
		double weight=0, matches, term_score; 
		int index;
		for (int ii=0;ii<topic.size();ii++){ //for each row of the topic
			//get the term
			tempstr=topic.get(ii);
			term = tempstr[1];
			matches=0;
			term_score=0;
			//find term in text
			Pattern pattern = Pattern.compile(" "+term+" ");	
			Matcher matcher = pattern.matcher(" "+str+" ");
			//list with positions of a found term
			ArrayList<String> termpos=new ArrayList<String>(); 
			while (matcher.find()) {
				//get the position of a found term
				termpos.add(Integer.toString(matcher.start()));
				matches++;
			}
			if (matches>0){
				//add found term
				termfound.add(term);
				//add id of the found term
				termfoundid.add(Integer.toString(ii+1));
				//get the weight of the term
				weight=Double.parseDouble(tempstr[0]);
				term_score = weight*matches;
				//get the subclass of the term
				term_class = tempstr[2];
				//add the "contribution" of the term to scores array, in the row 
				//which corresponds to the subclass  
				index = Arrays.binarySearch(classes, term_class);
				scores[index][0] =scores[index][0]+term_score;
				//print found terms and their positions for checking
				//System.out.println("Term :"+ term +" with termid:"+ (ii+1) +" in posistions: " +termpos);
			}
		}
		//calculate scores with the weight of the text (the location weight) 
		//and the relative scores
		for (int ii=0;ii<classes.length;ii++){
			scores[ii][0] = scores[ii][0]*w;
			scores[ii][1] = scores[ii][0]/words_num; 
			scores[classes.length][0] =scores[ii][0]+scores[classes.length][0];
			scores[classes.length][1] =scores[ii][1]+scores[classes.length][1];
		}
		//print results for checking
		//System.out.println("Num of words: "+words_num +" for location with weigth "+ w);
		//for (int ii=0;ii<scores.length;ii++) {
		//	System.out.println(scores[ii][0]+"    "+scores[ii][1]);
		//}
		return scores;
	}*/
	public static Double[][] rankText(String str, double w,ArrayList<String[]> topic, String[] classes, boolean isContent){
		//str: String to classify,  topic: list of triplets,  classes:list of classes
		//Returns an array of doubles with 2 columns. 
		//The number of rows are as many as the subclasses plus one (the "total"). 
		//The first contains the absolute score per subclass, while the second relative scores, 
		//i.e. the absolute scores divided by the number of words. 
		int uniqueTermsFound = 0;

		//stem the input str
		ArrayList<String> stems =new ArrayList<String>();
		try {
			stems = TopicTools.analyze(str, langIdentified);
		} catch (IOException e) {
			LOGGER.warn(e.getMessage());
			return null;
		}
		//concatenate stems 
		str="";
		for (String s:stems){ str+=" "+s;}
		str = str.trim();

		//System.out.println(str);
		//word counting
		StringTokenizer st = new StringTokenizer(str);	
		double words_num=st.countTokens();
		//double words_num = 0; String ss="";
		//while (st.hasMoreTokens()) { ss= st.nextToken();words_num++;}
		//initialization of scores array
		Double[][] scores = new Double[classes.length+1][2];
		for (int ii=0;ii<classes.length;ii++){
			scores[ii][0] = 0.0;	scores[ii][1] = 0.0;
		}		
		scores[classes.length][0]=0.0;  scores[classes.length][1]=0.0;
		if (words_num==0.0) return scores;
		//lists with the found terms
		//ArrayList<String> termfound=new ArrayList<String>(); 
		//ArrayList<String> termfoundid=new ArrayList<String>();
		//calculate the initial scores based on the occurrences of each term of the topic 
		//in the str and the weight of the term
		String[] tempstr = new String[1];		String term, term_class, term_lang;
		double weight=0, matches, term_score; 
		int index;
		for (int ii=0;ii<topic.size();ii++){ //for each row of the topic
			//get the term
			tempstr=topic.get(ii);
			term = tempstr[1];
			term_lang=tempstr[3];
			matches=0;
			term_score=0;
			weight=Double.parseDouble(tempstr[0]);
			//find term in text
			if (!term_lang.equals(langIdentified))
				continue;
			Pattern pattern = Pattern.compile(" "+term+" ");	
			Matcher matcher = pattern.matcher(" "+str+" ");
			//list with positions of a found term
			ArrayList<String> termpos=new ArrayList<String>(); 
			while (matcher.find()) {
				//get the position of a found term
				termpos.add(Integer.toString(matcher.start()));
				matches++;
				
			}
			
			if (matches>0){
				//System.out.println(term+ " "+ matches);
				if (weight>0.0){
					uniqueTermsFound++;
					//System.out.println(term);
					//System.out.println(uniqueTermsFound);
				}
				//add found term
				//termfound.add(term);
				//add id of the found term
				//termfoundid.add(Integer.toString(ii+1));
				//get the weight of the term
				//weight=Double.parseDouble(tempstr[0]);
				term_score = weight*matches;
				//get the subclass of the term
				term_class = tempstr[2];
				if (term_class.contains(";")){
					String[] term_classes = term_class.split(";");
					int term_classes_amount = term_classes.length;
					for (int mm=0;mm<term_classes_amount;mm++){
						index = Arrays.binarySearch(classes, term_classes[mm]);
						scores[index][0] =scores[index][0]+term_score/term_classes_amount;
					}
				}
				else{
					index = Arrays.binarySearch(classes, term_class);
					scores[index][0] =scores[index][0]+term_score;
				}
				//add the "contribution" of the term to scores array, in the row 
				//which corresponds to the subclass  
				//index = Arrays.binarySearch(classes, term_class);
				//scores[index][0] =scores[index][0]+term_score;
				//print found terms and their positions for checking				
			}
		}
		//Check if found terms in the content are equal or above the predefined threshold
		if (isContent) {
			if (uniqueTermsFound<_min_uniq_terms && _min_uniq_terms>0) {
				//for (int ii=0;ii<classes.length;ii++){
				//	scores[ii][0] =-1000000.0;	scores[ii][1] = -1000000.0;
				//}
				//return scores;
				return null;
			}
		}
		//calculate scores with the weight of the text (the location weight) 
		//and the relative scores
		for (int ii=0;ii<classes.length;ii++){
			scores[ii][0] = scores[ii][0]*w;
			if (isContent)
				scores[ii][1] = scores[ii][0]/words_num;
			scores[classes.length][0] =scores[ii][0]+scores[classes.length][0];
			scores[classes.length][1] =scores[ii][1]+scores[classes.length][1];			
		}		
		return scores;
	}

	//vpapa comment it
	/*public double rankLink(String text1) {
		double score = 0, weight=0; int matches=0;
		String[] tempstr = null;		
		String term;
		String text = text1.trim();
		System.out.println(text);
		ArrayList<String> stems =new ArrayList<String>();
		try {
			//stems = TopicTools.analyze(text, _targetLanguage);
			//vpapa
			String[] langs = _targetLanguage.split(";");
			//score+=Bitexts.heuristitlinkscore(text,langs);
			String langidentified_link="";
			System.out.println("langidentified_page:"+langIdentified);
			if (!text.isEmpty()){
				langidentified_link= checkLang(text);
				System.out.println("langidentified_link:"+langidentified_link);
			}
			else
				return score;
			//vpapa
			boolean match = false;
			int m=0;
			for (String lang:langs){
				//if (text.equals(lang)){
				if (containLangKeys(text,m)){
					match = true;
					System.out.println("BINGO");
					langidentified_link=lang;
					break;
				}
				if (langidentified_link.equals(lang)) {
					match = true;
					//break;
				}
				m++;
			}
			boolean match1 = false;
			for (String lang:langs){
				if (langIdentified.equals(lang)){
					match1=true;
					break;
				}
			}
			//vpapa
			if (match){
				if (langidentified_link.equals(langIdentified)){
					System.out.println(langidentified_link +" "+langIdentified);
					score += _thres;
				}
				else{
					if (match1){
						System.out.println(langidentified_link +" "+langIdentified);
						score += 10*_thres;
						//System.out.println(score);
					}
				}
				stems = TopicTools.analyze(text, langidentified_link);
			}else {
				return score;
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		text="";
		for (String s:stems){ text=text.concat(" "+s);}
		text = text.trim();
		for (int ii=0;ii<_topic.size();ii++){ 
			tempstr=_topic.get(ii);
			term = tempstr[1];
			matches=0;
			Pattern pattern = Pattern.compile(" "+term+" ");
			Matcher matcher = pattern.matcher(" "+text+" ");
			while (matcher.find()) {
				matches++;
			}
			if (matches>0){
				weight=Double.parseDouble(tempstr[0]);
				score += weight*matches;
				//System.out.println("FoundTerm :"+ term );
			}
		}
		return score;
	}*/


	/*public double rankLinktest(String text1) {
		double score = 0, weight=0; int matches=0;
		String[] tempstr = null;		
		String term;
		String text = text1.trim();
		//		System.out.println(text);
		ArrayList<String> stems =new ArrayList<String>();
		try {
			//vpapa
			String[] langs = _targetLanguage.split(";");
			boolean matchL=false;
			//boolean matchT=false, matchL=false;
			//for (String lang:langs){
			//	if (langIdentified.equals(lang)){
			//		matchT=true; //current page is in one of the targeted languages 
			//		break;
			//	}
			//}
			String langidentified_link="";
			//System.out.println("langidentified_page:"+langIdentified);
			if (!text.isEmpty()){
				langidentified_link= checkLang(text);
				//System.out.println("langidentified_link:"+langidentified_link);
				for (String lang:langs){
					if (langidentified_link.equals(lang)){
						matchL=true; //text of current link is in one of the targeted languages 
						break;
					}
				}
			}else
				return score;
			int type=0;
			if (langs.length>1){
				int m=0;
				for (String lang:langs){
					if (containLangKeys(text,m) & !lang.equals(langIdentified)){
						//System.out.println("BINGO!");
						//link's text implies that the link points to candidate translation
						type=1;
						break;
					}
					m++;
				}
				if (type==1){
					//link's text implies that the link points to candidate translation
					score +=1000*_thres;
					return score;
				}
			}
			//if (matchT & matchL){
			if (matchL){
				//if (!langidentified_link.equals(langIdentified)){
				//	score+=10*_thres; //text and link are in dif. langs but both in targ. langs
				//}else{
				score+=_thres;  //text and link are in same langs and in targ. langs
				//}
				stems = TopicTools.analyze(text, langidentified_link);
			}
			else
				return score;
		} catch (IOException e) {
			e.printStackTrace();
		}
		text="";
		for (String s:stems){ text=text.concat(" "+s);}
		text = text.trim();
		for (int ii=0;ii<_topic.size();ii++){ 
			tempstr=_topic.get(ii);
			term = tempstr[1];
			matches=0;
			Pattern pattern = Pattern.compile(" "+term+" ");
			Matcher matcher = pattern.matcher(" "+text+" ");
			while (matcher.find()) {
				matches++;
			}
			if (matches>0){
				weight=Double.parseDouble(tempstr[0]);
				score += weight*matches;
				//System.out.println("FoundTerm :"+ term );
			}
		}
		return score;
	}

*/






	public double rankLink(String text1, String anchortext, String pagelang, double vv) {
		double score = 0, weight=0; int matches=0;
		String[] tempstr = null;		
		String term;
		String text = text1.trim();
		//System.out.print(text);
		ArrayList<String> stems =new ArrayList<String>();

		try {
			//vpapa
			String[] langs = _targetLanguage.split(";");
			boolean matchT=false, matchL=false;
			for (String lang:langs){
				if (pagelang.equals(lang)){
					matchT=true; //current page is in one of the targeted languages 
					break;
				}
			}
			String langidentified_link="";
			//System.out.println("langidentified_page:"+langIdentified);

			if (!text.isEmpty()){
				langidentified_link= checkLang(text.toLowerCase());
				//System.out.print("langidentified_link:"+langidentified_link+"\t");
				for (String lang:langs){
					if (langidentified_link.equals(lang)){
						matchL=true; //text of current link is in one of the targeted languages 
						break;
					}
				}
			}else
				return score;
			int type=0;
			if (langs.length>1){
				int m=0;
				for (String lang:langs){
					//if (containLangKeys(text,m) & !lang.equals(pagelang)){
					if (containLangKeys(anchortext,m) & !lang.equals(pagelang)){
						//System.out.println("BINGO!");
						//link's text implies that the link points to candidate translation
						type=1;
						break;
					}
					m++;
				}
				if (type==1 & vv>=_absthres){
					//link's text implies that the link points to candidate translation
					score +=10000*_absthres;
					return score;
				}
			}
			if (matchT & matchL){
				StringTokenizer tkzr = new StringTokenizer(text);
				if (!langidentified_link.equals(pagelang) & tkzr.countTokens()>3 & vv>=_absthres){
					score+=10*_absthres; //text and link are in dif. langs but both in targ. langs
				}else{
					score+=_absthres;  //text and link are in same langs and in targ. langs
				}
				stems = TopicTools.analyze(text, langidentified_link);
			}
			else
				return score;
		} catch (IOException e) {
			e.printStackTrace();
		}
		text="";
		for (String s:stems){ text=text.concat(" "+s);}
		text = text.trim();
		for (int ii=0;ii<_topic.size();ii++){ 
			tempstr=_topic.get(ii);
			term = tempstr[1];
			matches=0;
			Pattern pattern = Pattern.compile(" "+term+" ");
			Matcher matcher = pattern.matcher(" "+text+" ");
			while (matcher.find()) {
				matches++;
			}
			if (matches>0){
				weight=Double.parseDouble(tempstr[0]);
				score += weight*matches;
				//System.out.println("FoundTerm :"+ term );
			}
		}
		return score;
	}

	//vpapa
	private boolean containLangKeys(String text, int m) {
		boolean result = false;
		if  (_targetlangKeys[m].isEmpty())
			return result;
		String[] keys= _targetlangKeys[m].split(",");
		String[] words= text.toLowerCase().split(" ");
		if (words.length>10)
			return result;
		for (int ii=0;ii<keys.length;ii++){
			for (int jj=0;jj<words.length;jj++){
				if (keys[ii].equals(words[jj])){
					result=true;
					break;
				}
			}
		}
		return result;
	}

	public static String cleanContent(String content){
		String result = "";
		//vpapa
		String REGEX = "<text.*>.*</text>";
		String REPLACE = " ";
		Pattern p = Pattern.compile(REGEX);
		Matcher m = p.matcher(content);
		String text = "";
		while (m.find()){
			text = m.group();
			//vpapa
			text = text.replaceAll(" type='listelem'","");
			text = text.replaceAll(" type='title'","");
			text = text.replaceAll(" type='heading'","");
			text = text.replaceAll("</?text>", REPLACE);
			result = result.concat(text);
		}
		return result;
	}

	public ArrayList<String[]> getTopic(){
		return _topic;
	}

	public int getMaxDepth() { return _max_depth;}


	
	public static String checkLang(String partOfLine) {
		String langidentified ="";
		if (partOfLine.length()<5)
			return langidentified;
		//LanguageIdentifier LangI=new LanguageIdentifier(partOfLine); 
		//langidentified = LangI.getLanguage();
		//if (!langidentified.equals(targetLang)){
		Detector detector = null;			
		try {
			detector = DetectorFactory.create();
			detector.append(partOfLine);
			langidentified = detector.detect();										
		} catch (LangDetectException e) {
			//LOGGER.error(e.getMessage());
		}
		//}
		return langidentified;
	}

	/*public double rankLink1(String url) {
		double score = 0;
		String[] langs = _targetLanguage.split(";");
		if (langs.length>1){
			if (url.contains("lang=")){
				score=100*_thres;
			}
		}else
			score=0;
		return score;
	}*/
	public double rankLinkNotopic(String linktext, String anchortext, String pagelang, double vv) {
		double score = 0;
				
		
		String text = linktext.trim();
		//System.out.print(text);
		//vpapa
		String[] langs = _targetLanguage.split(";");
		boolean matchT=false, matchL=false;
		for (String lang:langs){
			if (pagelang.equals(lang)){
				matchT=true; //current page is in one of the targeted languages 
				break;
			}
		}
		String langidentified_link="";
		//System.out.println("langidentified_page:"+langIdentified);

		if (!text.isEmpty()){
			langidentified_link= checkLang(text.toLowerCase());
			//System.out.print("langidentified_link:"+langidentified_link+"\t");
			for (String lang:langs){
				if (langidentified_link.equals(lang)){
					matchL=true; //text of current link is in one of the targeted languages 
					break;
				}
			}
		}else
			return score;
		int type=0;
		if (langs.length>1){
			int m=0;
			for (String lang:langs){
				//if (containLangKeys(text,m) & !lang.equals(pagelang)){
				if (containLangKeys(anchortext,m) & !lang.equals(pagelang)){
					//System.out.println("BINGO!");
					//link's text implies that the link points to candidate translation
					type=1;
					break;
				}
				m++;
			}
			if (type==1 & vv>=_absthres){
				//link's text implies that the link points to candidate translation
				score +=2;
				return score;
			}
		}
		if (matchT & matchL){
			StringTokenizer tkzr = new StringTokenizer(text);
			if (!langidentified_link.equals(pagelang) & tkzr.countTokens()>3 & vv>=_absthres){
				score+=1; //text and link are in dif. langs but both in targ. langs
			}else{
				score+=1;  //text and link are in same langs and in targ. langs
			}
		}
		else
			return score;
		return score;
	}
}
